#!/usr/bin/env python
# coding=utf-8

import os
import sys
from pathlib import Path

from flask import Flask, request, Response, render_template as rt

from .utils.get_folder_info import get_file_and_folder
from .utils.make_qrcode import get_inner_ip, open_browser, make_qrcode_
from .utils.health_examination import net_is_used

app = Flask(__name__)

# 项目根路径
BASE_DIR = os.path.dirname(os.path.abspath(__file__))

# 上传路径
upload = os.path.join(BASE_DIR, 'upload/{}')

# 全局内网IP
global_inner_ip = 'test'


@app.route('/', methods=['GET'])
def index():
    return rt('./index_new.html', global_inner_ip=global_inner_ip)


@app.route('/file/upload', methods=['POST'])
def upload_part():  # 接收前端上传的一个分片
    task = request.form.get('task_id')  # 获取文件的唯一标识符
    chunk = request.form.get('chunk', 0)  # 获取该分片在所有分片中的序号
    filename = '%s%s' % (task, chunk)  # 构造该分片的唯一标识符

    upload_file = request.files['file']
    upload_file.save(upload.format(filename))  # 保存分片到本地
    return rt('./index.html')


@app.route('/file/merge', methods=['GET'])
def upload_success():  # 按序读出分片内容，并写入新文件
    target_filename = request.args.get('filename')  # 获取上传文件的文件名
    task = request.args.get('task_id')  # 获取文件的唯一标识符
    chunk = 0  # 分片序号
    path = upload.format(target_filename)
    with open(path, 'wb') as target_file:  # 创建新文件
        while True:
            try:
                filename = upload.format('{}{}'.format(task, chunk))
                source_file = open(filename, 'rb')  # 按序打开每个分片
                target_file.write(source_file.read())  # 读取分片内容写入新文件
                source_file.close()
            except IOError as msg:
                break
            chunk += 1
            os.remove(filename)  # 删除该分片，节约空间

    return rt('./index.html')


@app.route('/file/list', methods=['GET'])
def file_list():
    folder = request.args.get('folder', '')
    pre_folder = request.args.get('pre_folder')
    # 获取上传路劲以下子路径
    base_path = upload.replace('\\', '/').format('')
    if pre_folder:
        folder_list = pre_folder.replace(base_path, '').split('/')
        try:
            folder = folder_list[-2]
        except Exception as e:
            folder = ''
    path = upload.format(folder)
    if folder and not Path(path).exists():
        path = upload.format('')
    path = path.replace('\\', '/')
    # 获取文件目录
    files = get_file_and_folder(path)
    upload_path = upload.format('').replace('\\', '/')
    # files = map(lambda x: x if isinstance(x, 'unicode') else x.decode('utf-8'), files)  # 注意编码
    return rt('./list_new.html', files=files, path=path, folder=folder, upload_path=upload_path)


@app.route('/file/download/<filename>', methods=['GET'])
def file_download(filename):
    def send_chunk():  # 流式读取
        store_path = upload.format(filename)
        with open(store_path, 'rb') as target_file:
            while True:
                chunk = target_file.read(20 * 1024 * 1024)
                if not chunk:
                    break
                yield chunk

    return Response(send_chunk(), content_type='application/octet-stream')


def main(**kwargs):
    host = kwargs.get('host') if kwargs.get('host') else '0.0.0.0'
    port = int(kwargs.get('port')) if kwargs.get('port') else 5000
    # 检查 上传目录是否存在 不存在就创建
    if not Path(upload.format('')).exists():
        os.mkdir(upload.format(''))
    if net_is_used(port):
        for i in range(3):
            port += 1
            if not net_is_used(port):
                break
    # 生成二维码
    inner_ip = get_inner_ip()
    global global_inner_ip
    global_inner_ip = inner_ip
    make_url = 'http://{}:{}'.format(inner_ip, port)
    save_path = os.path.join(BASE_DIR, 'static/images/qrcode/')
    make_qrcode_(make_url=make_url, save_path=save_path, qrcode_name='{}.png'.format(inner_ip))
    # 自动打开浏览器
    if kwargs.get('open_browser', True):
        open_url = 'http://{}:{}'.format(inner_ip, port)
        open_browser(open_url)
    app.run(debug=False, threaded=True, host=host, port=port)

# -----------------------------------------------------------------------------
# Main entry point
# -----------------------------------------------------------------------------


if __name__ == '__main__' or '__main__' in sys.argv:
    main()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   health_examination.py    
@Contact :   https://2409256477@qq.com
@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
2021/2/7 13:15   huangjh      1.0         None
"""
import socket


def net_is_used(port, ip='127.0.0.1'):
    """
    查看端口是否占用
    :param port: 端口
    :param ip:
    :return: boolean False 表示没有占用 True 表示已被占用
    """
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        s.connect((ip, port))
        s.shutdown(2)
        return True
    except Exception as e:
        print(e)
        return False


if __name__ == '__main__':
    # False 表示没有占用
    is_ok = net_is_used(5000)
    print(is_ok)



import os
from pathlib import Path

import qrcode
import socket
import webbrowser
import time


def get_inner_ip():
    """
    获取本机内网ip
    :return:
    """
    return socket.gethostbyname(socket.gethostname())


def open_browser(open_url='http://www.baidu.com'):
    """
    在浏览器中打开指定url
    :return:
    """
    webbrowser.open_new(open_url)


def make_qrcode_(make_url='http://101.200.206.164/', save_path='./', qrcode_name='test.png'):
    """
    生成二维码
    :param make_url: 需要生成的url地址
    :param save_path: 保存路径
    :param qrcode_name: 保存图面名称
    :return:
    """
    # img_path = qrcode.make(make_url)
    # with open(os.path.join(save_path, qrcode_name), 'wb') as f:
    #     img_path.save(f)
    qrcode_img = os.path.join(save_path, qrcode_name)
    if Path(qrcode_img).exists():
        return

    # 删除其他二维码
    files = os.listdir(save_path)
    for f in files:
        os.remove(os.path.join(save_path, f))

    # 高级用法
    qr = qrcode.QRCode(
        # 二维码矩阵尺寸
        version=1,
        # 二维码容错率
        error_correction=qrcode.constants.ERROR_CORRECT_H,
        # 二维码中每个Box的像素值
        box_size=10,
        # 二维码与图片边界的距离,默认为4
        border=0,
    )
    qr.add_data(make_url)
    qr.make(fit=True)

    img1 = qr.make_image()
    img1.save(os.path.join(save_path, qrcode_name))


if __name__ == '__main__':
    # make_qrcode()
    # print(get_inner_ip())
    url = 'http://' + get_inner_ip() + ':8002'
    open_browser(open_url=url)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   get_folder_info.py    
@Contact :   https://2409256477@qq.com
@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
2021/2/6 10:20   huangjh      1.0         None
"""
import datetime
import os
import time


def get_file_and_folder(target_path):
    """
    获取文件夹下文件 文件夹信息
    :param target_path: 目标路径
    :return:
    """
    files = os.listdir(target_path)
    result_list = list()
    for f in files:
        file_path = os.path.join(target_path, f)
        # 获取时间
        file_update = os.stat(target_path).st_mtime
        file_update = time_format(file_update)
        file_create = os.stat(target_path).st_ctime
        file_create = time_format(file_create)
        file_dict = {'crete_date': file_create, 'update_date': file_update}
        # 判断文件/文件夹
        if os.path.isdir(file_path):
            file_dict.update({'name': f, 'is_folder': True})
        elif os.path.isfile(file_path):
            file_size = os.path.getsize(os.path.join(target_path, f))
            file_size = size_format(file_size)
            file_dict.update({'name': f, 'size': file_size, 'is_folder': False})
        else:
            continue
        result_list.append(file_dict)
    return result_list


def time_format(timestamp, format_str="%Y-%m-%d %H:%M:%S"):
    """
    获取固定格式时间戳
    :param timestamp:
    :param format_str:
    :return:
    """
    date_array = datetime.datetime.fromtimestamp(timestamp)
    format_date = date_array.strftime(format_str)
    return format_date


def size_format(size):
    """
    文件大小格式化
    :param size: byte
    :return:
    """
    if size < 1000:
        return '%i' % size + 'size'
    elif 1000 <= size < 1000000:
        return '%.1f' % float(size/1000) + 'KB'
    elif 1000000 <= size < 1000000000:
        return '%.1f' % float(size/1000000) + 'MB'
    elif 1000000000 <= size < 1000000000000:
        return '%.1f' % float(size/1000000000) + 'GB'
    elif 1000000000000 <= size:
        return '%.1f' % float(size/1000000000000) + 'TB'


if __name__ == '__main__':
    path_ = 'D:\\program\\pyupload\\upload/'
    get_file_and_folder(path_)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   setup.py    
@Contact :   https://2409256477@qq.com
@Modify Time      @Author    @Version    @Desciption
------------      -------    --------    -----------
2021/2/6 21:37   huangjh      1.0         None
"""
import setuptools

with open("README.md", "r", encoding="UTF-8") as fh:
    long_description = fh.read()

install_requires = ['click', 'Flask', 'itsdangerous', 'Jinja2', 'MarkupSafe', 'Pillow', 'qrcode',
                    'six', 'Werkzeug']

setuptools.setup(
    name="fast-ft",
    version="0.0.9",
    author="Uncle supported wall",
    author_email="2409256477@qq.com",
    description="A simple file transfer tool",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/practice9420/fast-ft",
    packages=setuptools.find_packages(),
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=install_requires,
    entry_points={
        'console_scripts': [
            'fast-ft = fast_ft.server:main',
        ]
    }
)
